/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SableCC.                             *
 * See the file "LICENSE" for copyright information and the  *
 * terms and conditions for copying, distribution and        *
 * modification of SableCC.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package org.sablecc.sablecc;

import org.sablecc.sablecc.analysis.*;
import org.sablecc.sablecc.node.*;
import java.util.*;

public class Transitions extends DepthFirstAdapter
{
  public final Map<ATokenDef, Map<String, String>> tokenStates = new HashMap<>();

  private String state;
  private String transition;
  private Map<String, String> map;

  @Override
  public void caseAStateList(AStateList node)
  {
    inAStateList(node);
    if(node.getId() != null)
    {
      node.getId().apply(this);
    }
    if(node.getTransition() != null)
    {
      node.getTransition().apply(this);
    }

    outAStateList(node);  // We moved this...

    {
      PStateListTail[] temp = node.getStateLists().toArray(new PStateListTail[0]);
      for(PStateListTail tail : temp)
      {
        tail.apply(this);
      }
    }
  }

  @Override
  public void inATokenDef(ATokenDef node)
  {
    map = new TreeMap<>();
  }

  @Override
  public void inAStateList(AStateList node)
  {
    state = transition = node.getId().getText().toUpperCase();
  }

  @Override
  public void inAStateListTail(AStateListTail node)
  {
    state = transition = node.getId().getText().toUpperCase();
  }

  @Override
  public void outATransition(ATransition node)
  {
    transition = node.getId().getText().toUpperCase();
  }

  @Override
  public void outAStateList(AStateList node)
  {
    map.put(state, transition);
  }

  @Override
  public void outAStateListTail(AStateListTail node)
  {
    map.put(state, transition);
  }

  @Override
  public void outATokenDef(ATokenDef node)
  {
    tokenStates.put(node, map);
  }
}
